package apap.tutorial.belajarbelajar.restController;

import java.util.HashMap;
import java.util.List;
import java.util.NoSuchElementException;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.server.ResponseStatusException;

import apap.tutorial.belajarbelajar.model.PengajarModel;
import apap.tutorial.belajarbelajar.service.PengajarRestService;
import apap.tutorial.belajarbelajar.service.CourseRestService;

@RestController
@RequestMapping("/api/v1")
public class PengajarRestController {
    @Autowired
    private PengajarRestService pengajarRestService;

    //Add
    @PostMapping(value = "/pengajar")
    private PengajarModel createPengajar(@Valid @RequestBody PengajarModel pengajar, BindingResult bindingResult){
        if (bindingResult.hasFieldErrors()){
            throw new ResponseStatusException(
                HttpStatus.BAD_REQUEST, "Request body has invalid type or missing field."
            );
        } else {
            return pengajarRestService.createPengajar(pengajar);
        }
    }

    //Update
    @PutMapping(value = "/pengajar/{noPengajar}")
    private PengajarModel updatePengajar(@PathVariable("noPengajar") Long noPengajar, @RequestBody PengajarModel pengajar){
        try {
            return pengajarRestService.updatePengajar(noPengajar, pengajar);
        } catch (NoSuchElementException e){
            throw new ResponseStatusException(
                HttpStatus.NOT_FOUND, "Nomor pengajar " + noPengajar + " not found"
            );
        }
    }

    //Retrieve
    @GetMapping(value = "/pengajar/{noPengajar}")
    private PengajarModel retrievePengajar(@PathVariable("noPengajar") Long noPengajar){
        try {
            return pengajarRestService.getPengajarByNoPengajar(noPengajar);
        } catch (NoSuchElementException e){
            throw new ResponseStatusException(
                HttpStatus.NOT_FOUND, "Nomor pengajar " + noPengajar + " not found"
            );
        }
    }

    //Retrieve List All
    @GetMapping(value = "/list-pengajar")
    private List<PengajarModel> retrieveListPengajar(){
        return pengajarRestService.retrieveListPengajar();
    }

    //Delete
    @DeleteMapping(value = "/pengajar/{noPengajar}")
    private ResponseEntity deletePengajar(@PathVariable("noPengajar") Long noPengajar){
        try {
            pengajarRestService.deletePengajar(noPengajar);
            return ResponseEntity.ok("Pengajar with no pengajar " + noPengajar + " has been deleted successfully");
        } catch (NoSuchElementException e){
            throw new ResponseStatusException(
                HttpStatus.NOT_FOUND, "No pengajar " + noPengajar + " not found"
            );
        }
    }

    @GetMapping(value = "/pengajar/jenis-kelamin/{noPengajar}")
    private PengajarModel predictGender(@PathVariable("noPengajar") Long noPengajar) {
        try {
            PengajarModel pengajar = pengajarRestService.getPengajarByNoPengajar(noPengajar);
            if (pengajarRestService.isClosed(pengajar.getCourse().getTanggalDimulai(), pengajar.getCourse().getTanggalBerakhir())){
                    String[] name = pengajar.getNamaPengajar().split(" ");
                    String url = String.format("https://api.genderize.io/?name=%s", name[0]);
                    RestTemplate restTemplate = new RestTemplate();
                    HashMap<String, String> result = restTemplate.getForObject(url, HashMap.class);
                    if (result.get("gender").equals("female")) {
                        return pengajarRestService.predictGender(noPengajar, true);
                    } else {
                        return pengajarRestService.predictGender(noPengajar, false);
                    }
            } else {
                throw new UnsupportedOperationException("Course masih dibuka");
            }
        } catch (NoSuchElementException e){
            throw new ResponseStatusException(
                HttpStatus.NOT_FOUND, "Nomor pengajar " + noPengajar + " not found"
            );
        }
    }
}
