package apap.tutorial.belajarbelajar.service;
import java.time.LocalDateTime;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Optional;

import javax.transaction.Transactional;

import apap.tutorial.belajarbelajar.rest.Setting;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.querydsl.binding.MultiValueBinding;
import org.springframework.stereotype.Service;

import apap.tutorial.belajarbelajar.model.PengajarModel;
import apap.tutorial.belajarbelajar.repository.PengajarDb;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

@Service
@Transactional
public class PengajarRestServiceImpl implements PengajarRestService{
    private final WebClient webClient;
    
    public PengajarRestServiceImpl(WebClient.Builder webClientBuilder) {
        this.webClient = webClientBuilder.baseUrl(Setting.courseUrl).build();
    }

    @Autowired
    private PengajarDb pengajarDb;
    
    @Override
    public PengajarModel createPengajar(PengajarModel pengajar) {
        return pengajarDb.save(pengajar);
    }

    @Override
    public PengajarModel updatePengajar(Long noPengajar, PengajarModel pengajarUpdate) {
        PengajarModel pengajar = getPengajarByNoPengajar(noPengajar);
        pengajar.setNamaPengajar(pengajarUpdate.getNamaPengajar());
        pengajar.setIsPengajarUniversitas(pengajarUpdate.getIsPengajarUniversitas());
        return pengajarDb.save(pengajar);
    }

    @Override
    public PengajarModel getPengajarByNoPengajar(Long noPengajar) {
        Optional<PengajarModel> pengajar = pengajarDb.findByNoPengajar(noPengajar);
        if (pengajar.isPresent()) {
            return pengajar.get();
        } else {
            throw new NoSuchElementException();
        }
    }

    @Override
    public List<PengajarModel> retrieveListPengajar() {
        return pengajarDb.findAll();
    }

    @Override
    public void deletePengajar(Long noPengajar) {
        PengajarModel pengajar = getPengajarByNoPengajar(noPengajar);
        pengajarDb.delete(pengajar);
    }

    @Override
    public boolean isClosed(LocalDateTime tanggalDimulai, LocalDateTime tanggalBerakhir) {
        LocalDateTime now = LocalDateTime.now();
        if (now.isBefore(tanggalDimulai) || now.isAfter(tanggalBerakhir)) {
            return true;
        }
        return false;
    }

    @Override
    public PengajarModel predictGender(Long noPengajar, Boolean b){
        PengajarModel pengajar = getPengajarByNoPengajar(noPengajar);
        pengajar.setJenisKelamin(b);
        return pengajarDb.save(pengajar);
    }
}
